@@ -86,17 +86,18 @@ impl From<GraphWidthType> for graph::CellWidthType {
     }
 }
 
-pub fn run() -> std::io::Result<()> {
-    color_eyre::install().unwrap();
+pub type Result<T> = std::result::Result<T, Box<dyn std::error::Error>>;
+
+pub fn run() -> Result<()> {
     let args = Args::parse();
-    let (ui_config, graph_config, key_bind_patch) = config::load();
+    let (ui_config, graph_config, key_bind_patch) = config::load()?;
     let key_bind = keybind::KeyBind::new(key_bind_patch);
 
     let color_theme = color::ColorTheme::default();
     let graph_color_set = color::GraphColorSet::new(&graph_config.color);
     let image_protocol = args.protocol.into();
 
-    let repository = git::Repository::load(Path::new("."), args.order.into());
+    let repository = git::Repository::load(Path::new("."), args.order.into())?;
 
     let graph = graph::calc_graph(&repository);
 
@@ -130,5 +131,5 @@ pub fn run() -> std::io::Result<()> {
     let ret = app.run(&mut terminal, rx);
 
     ratatui::restore();
-    ret
+    ret.map_err(Into::into)
 }
