@@ -42,6 +42,14 @@ impl<'a> ListView<'a> {
                     self.as_mut_list_state().cancel_search();
                     self.clear_search_query();
                 }
+                UserEvent::IgnoreCaseToggle => {
+                    self.as_mut_list_state().toggle_ignore_case();
+                    self.update_search_query();
+                }
+                UserEvent::FuzzyToggle => {
+                    self.as_mut_list_state().toggle_fuzzy();
+                    self.update_search_query();
+                }
                 _ => {
                     self.as_mut_list_state().handle_search_input(key);
                     self.update_search_query();
@@ -158,11 +166,17 @@ impl<'a> ListView<'a> {
     }
 
     fn update_search_query(&self) {
-        let list_state = self.as_list_state();
-        if let Some(query) = list_state.search_query_string() {
-            let cursor_pos = list_state.search_query_cursor_position();
-            self.tx
-                .send(AppEvent::UpdateStatusInput(query, Some(cursor_pos)));
+        if let SearchState::Searching { .. } = self.as_list_state().search_state() {
+            let list_state = self.as_list_state();
+            if let Some(query) = list_state.search_query_string() {
+                let cursor_pos = list_state.search_query_cursor_position();
+                let transient_msg = list_state.transient_message_string();
+                self.tx.send(AppEvent::UpdateStatusInput(
+                    query,
+                    Some(cursor_pos),
+                    transient_msg,
+                ));
+            }
         }
     }
 
