@@ -1,6 +1,8 @@
 use std::collections::HashMap;
 
+use fuzzy_matcher::{skim::SkimMatcherV2, FuzzyMatcher};
 use laurier::highlight::highlight_matched_text;
+use once_cell::sync::Lazy;
 use ratatui::{
     buffer::Buffer,
     crossterm::event::{Event, KeyEvent},
@@ -18,6 +20,8 @@ use crate::{
     graph::GraphImageManager,
 };
 
+static FUZZY_MATCHER: Lazy<SkimMatcherV2> = Lazy::new(|| SkimMatcherV2::default().respect_case());
+
 const ELLIPSIS: &str = "...";
 
 #[derive(Debug)]
@@ -43,9 +47,11 @@ pub enum SearchState {
     Searching {
         start_index: usize,
         match_index: usize,
+        ignore_case: bool,
+        fuzzy: bool,
+        transient_message: TransientMessage,
     },
     Applied {
-        start_index: usize,
         match_index: usize,
         total_match: usize,
     },
@@ -61,8 +67,18 @@ impl SearchState {
     }
 }
 
-#[derive(Debug, Default, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum TransientMessage {
+    None,
+    IgnoreCaseOff,
+    IgnoreCaseOn,
+    FuzzyOff,
+    FuzzyOn,
+}
+
+#[derive(Debug, Default, Clone)]
 struct SearchMatch {
+    refs: HashMap<String, SearchMatchPosition>,
     subject: Option<SearchMatchPosition>,
     author_name: Option<SearchMatchPosition>,
     commit_hash: Option<SearchMatchPosition>,
@@ -70,26 +86,95 @@ struct SearchMatch {
 }
 
 impl SearchMatch {
+    fn new(c: &Commit, refs: &[&Ref], q: &str, ignore_case: bool, fuzzy: bool) -> Self {
+        let matcher = SearchMatcher::new(q, ignore_case, fuzzy);
+        let refs = refs
+            .iter()
+            .filter(|r| !matches!(*r, Ref::Stash { .. }))
+            .filter_map(|r| {
+                matcher
+                    .matched_position(r.name())
+                    .map(|pos| (r.name().into(), pos))
+            })
+            .collect();
+        let subject = matcher.matched_position(&c.subject);
+        let author_name = matcher.matched_position(&c.author_name);
+        let commit_hash = matcher.matched_position(&c.commit_hash.as_short_hash());
+        Self {
+            refs,
+            subject,
+            author_name,
+            commit_hash,
+            match_index: 0,
+        }
+    }
+
     fn matched(&self) -> bool {
-        self.subject.is_some() || self.author_name.is_some() || self.commit_hash.is_some()
+        !self.refs.is_empty()
+            || self.subject.is_some()
+            || self.author_name.is_some()
+            || self.commit_hash.is_some()
     }
 
     fn clear(&mut self) {
+        self.refs.clear();
         self.subject = None;
         self.author_name = None;
         self.commit_hash = None;
     }
 }
 
-#[derive(Debug, Default, Clone, Copy)]
+#[derive(Debug, Default, Clone)]
 struct SearchMatchPosition {
-    start: usize,
-    end: usize,
+    matched_indices: Vec<usize>,
 }
 
 impl SearchMatchPosition {
-    fn new(start: usize, end: usize) -> Self {
-        Self { start, end }
+    fn new(matched_indices: Vec<usize>) -> Self {
+        Self { matched_indices }
+    }
+}
+
+struct SearchMatcher {
+    query: String,
+    ignore_case: bool,
+    fuzzy: bool,
+}
+
+impl SearchMatcher {
+    fn new(query: &str, ignore_case: bool, fuzzy: bool) -> Self {
+        let query = if ignore_case {
+            query.to_lowercase()
+        } else {
+            query.into()
+        };
+        Self {
+            query,
+            ignore_case,
+            fuzzy,
+        }
+    }
+
+    fn matched_position(&self, s: &str) -> Option<SearchMatchPosition> {
+        if self.fuzzy {
+            let result = if self.ignore_case {
+                FUZZY_MATCHER.fuzzy_indices(&s.to_lowercase(), &self.query)
+            } else {
+                FUZZY_MATCHER.fuzzy_indices(s, &self.query)
+            };
+            result
+                .map(|(_, indices)| indices)
+                .map(SearchMatchPosition::new)
+        } else {
+            let result = if self.ignore_case {
+                s.to_lowercase().find(&self.query)
+            } else {
+                s.find(&self.query)
+            };
+            result
+                .map(|p| (p..(p + self.query.len())).collect())
+                .map(SearchMatchPosition::new)
+        }
     }
 }
 
@@ -313,6 +398,9 @@ impl<'a> CommitListState<'a> {
             self.search_state = SearchState::Searching {
                 start_index: self.current_selected_index(),
                 match_index: 0,
+                ignore_case: false,
+                fuzzy: false,
+                transient_message: TransientMessage::None,
             };
             self.search_input.reset();
             self.clear_search_matches();
@@ -320,25 +408,33 @@ impl<'a> CommitListState<'a> {
     }
 
     pub fn handle_search_input(&mut self, key: KeyEvent) {
-        if let SearchState::Searching { start_index, .. } = self.search_state {
+        if let SearchState::Searching {
+            transient_message, ..
+        } = &mut self.search_state
+        {
+            *transient_message = TransientMessage::None;
+        }
+
+        if let SearchState::Searching {
+            start_index,
+            ignore_case,
+            fuzzy,
+            ..
+        } = self.search_state
+        {
             self.search_input.handle_event(&Event::Key(key));
-            self.update_search_matches();
+            self.update_search_matches(ignore_case, fuzzy);
             self.select_current_or_next_match_index(start_index);
         }
     }
 
     pub fn apply_search(&mut self) {
-        if let SearchState::Searching {
-            start_index,
-            match_index,
-        } = self.search_state
-        {
+        if let SearchState::Searching { match_index, .. } = self.search_state {
             if self.search_input.value().is_empty() {
                 self.search_state = SearchState::Inactive;
             } else {
                 let total_match = self.search_matches.iter().filter(|m| m.matched()).count();
                 self.search_state = SearchState::Applied {
-                    start_index,
                     match_index,
                     total_match,
                 };
@@ -354,6 +450,60 @@ impl<'a> CommitListState<'a> {
         }
     }
 
+    pub fn toggle_ignore_case(&mut self) {
+        if let SearchState::Searching {
+            ignore_case,
+            transient_message,
+            ..
+        } = &mut self.search_state
+        {
+            *ignore_case = !*ignore_case;
+            *transient_message = if *ignore_case {
+                TransientMessage::IgnoreCaseOn
+            } else {
+                TransientMessage::IgnoreCaseOff
+            };
+        }
+
+        if let SearchState::Searching {
+            start_index,
+            ignore_case,
+            fuzzy,
+            ..
+        } = self.search_state
+        {
+            self.update_search_matches(ignore_case, fuzzy);
+            self.select_current_or_next_match_index(start_index);
+        }
+    }
+
+    pub fn toggle_fuzzy(&mut self) {
+        if let SearchState::Searching {
+            fuzzy,
+            transient_message,
+            ..
+        } = &mut self.search_state
+        {
+            *fuzzy = !*fuzzy;
+            *transient_message = if *fuzzy {
+                TransientMessage::FuzzyOn
+            } else {
+                TransientMessage::FuzzyOff
+            };
+        }
+
+        if let SearchState::Searching {
+            start_index,
+            ignore_case,
+            fuzzy,
+            ..
+        } = self.search_state
+        {
+            self.update_search_matches(ignore_case, fuzzy);
+            self.select_current_or_next_match_index(start_index);
+        }
+    }
+
     pub fn search_query_string(&self) -> Option<String> {
         if let SearchState::Searching { .. } = self.search_state {
             let query = self.search_input.value();
@@ -390,20 +540,34 @@ impl<'a> CommitListState<'a> {
         self.search_input.visual_cursor() as u16 + 1 // add 1 for "/"
     }
 
-    fn update_search_matches(&mut self) {
-        let query = self.search_input.value();
+    pub fn transient_message_string(&self) -> Option<String> {
+        if let SearchState::Searching {
+            transient_message, ..
+        } = self.search_state
+        {
+            match transient_message {
+                TransientMessage::None => None,
+                TransientMessage::IgnoreCaseOn => Some("Ignore case: ON ".to_string()),
+                TransientMessage::IgnoreCaseOff => Some("Ignore case: OFF".to_string()),
+                TransientMessage::FuzzyOn => Some("Fuzzy match: ON ".to_string()),
+                TransientMessage::FuzzyOff => Some("Fuzzy match: OFF".to_string()),
+            }
+        } else {
+            None
+        }
+    }
+
+    fn update_search_matches(&mut self, ignore_case: bool, fuzzy: bool) {
+        let q = self.search_input.value();
         let mut match_index = 1;
         for (i, commit_info) in self.commits.iter().enumerate() {
-            let mut m = SearchMatch::default();
-            if let Some(pos) = commit_info.commit.subject.find(query) {
-                m.subject = Some(SearchMatchPosition::new(pos, pos + query.len()));
-            }
-            if let Some(pos) = commit_info.commit.author_name.find(query) {
-                m.author_name = Some(SearchMatchPosition::new(pos, pos + query.len()));
-            }
-            if let Some(pos) = commit_info.commit.commit_hash.as_short_hash().find(query) {
-                m.commit_hash = Some(SearchMatchPosition::new(pos, pos + query.len()));
-            }
+            let mut m = SearchMatch::new(
+                commit_info.commit,
+                commit_info.refs.as_slice(),
+                q,
+                ignore_case,
+                fuzzy,
+            );
             if m.matched() {
                 m.match_index = match_index;
                 match_index += 1;
@@ -618,7 +782,12 @@ impl CommitList<'_> {
         let items: Vec<ListItem> = self
             .rendering_commit_info_iter(state)
             .map(|(i, commit_info)| {
-                let mut spans = refs_spans(commit_info, state.head, self.color_theme);
+                let mut spans = refs_spans(
+                    commit_info,
+                    state.head,
+                    &state.search_matches[state.offset + i].refs,
+                    self.color_theme,
+                );
                 let ref_spans_width: usize = spans.iter().map(|s| s.width()).sum();
                 let max_width = max_width.saturating_sub(ref_spans_width);
                 let commit = commit_info.commit;
@@ -631,7 +800,7 @@ impl CommitList<'_> {
                     };
 
                     let sub_spans =
-                        if let Some(pos) = state.search_matches[state.offset + i].subject {
+                        if let Some(pos) = state.search_matches[state.offset + i].subject.clone() {
                             highlighted_spans(
                                 subject,
                                 pos,
@@ -665,17 +834,18 @@ impl CommitList<'_> {
                 } else {
                     commit.author_name.to_string()
                 };
-                let spans = if let Some(pos) = state.search_matches[state.offset + i].author_name {
-                    highlighted_spans(
-                        name,
-                        pos,
-                        self.color_theme.list_name_fg,
-                        self.color_theme,
-                        truncate,
-                    )
-                } else {
-                    vec![name.fg(self.color_theme.list_name_fg)]
-                };
+                let spans =
+                    if let Some(pos) = state.search_matches[state.offset + i].author_name.clone() {
+                        highlighted_spans(
+                            name,
+                            pos,
+                            self.color_theme.list_name_fg,
+                            self.color_theme,
+                            truncate,
+                        )
+                    } else {
+                        vec![name.fg(self.color_theme.list_name_fg)]
+                    };
                 self.to_commit_list_item(i, spans, state)
             })
             .collect();
@@ -690,17 +860,18 @@ impl CommitList<'_> {
             .rendering_commit_iter(state)
             .map(|(i, commit)| {
                 let hash = commit.commit_hash.as_short_hash();
-                let spans = if let Some(pos) = state.search_matches[state.offset + i].commit_hash {
-                    highlighted_spans(
-                        hash,
-                        pos,
-                        self.color_theme.list_hash_fg,
-                        self.color_theme,
-                        false,
-                    )
-                } else {
-                    vec![hash.fg(self.color_theme.list_hash_fg)]
-                };
+                let spans =
+                    if let Some(pos) = state.search_matches[state.offset + i].commit_hash.clone() {
+                        highlighted_spans(
+                            hash,
+                            pos,
+                            self.color_theme.list_hash_fg,
+                            self.color_theme,
+                            false,
+                        )
+                    } else {
+                        vec![hash.fg(self.color_theme.list_hash_fg)]
+                    };
                 self.to_commit_list_item(i, spans, state)
             })
             .collect();
@@ -769,6 +940,7 @@ impl CommitList<'_> {
 fn refs_spans<'a>(
     commit_info: &'a CommitInfo,
     head: &'a Head,
+    refs_matches: &'a HashMap<String, SearchMatchPosition>,
     color_theme: &'a ColorTheme,
 ) -> Vec<Span<'a>> {
     let refs = &commit_info.refs;
@@ -782,20 +954,30 @@ fn refs_spans<'a>(
         }
     }
 
-    let ref_spans: Vec<Span> = refs
+    let ref_spans: Vec<(Vec<Span>, &String)> = refs
         .iter()
         .filter_map(|r| match r {
             Ref::Branch { name, .. } => {
-                Some(Span::raw(name).fg(color_theme.list_ref_branch_fg).bold())
+                let fg = color_theme.list_ref_branch_fg;
+                Some((name, fg))
+            }
+            Ref::RemoteBranch { name, .. } => {
+                let fg = color_theme.list_ref_remote_branch_fg;
+                Some((name, fg))
+            }
+            Ref::Tag { name, .. } => {
+                let fg = color_theme.list_ref_tag_fg;
+                Some((name, fg))
             }
-            Ref::RemoteBranch { name, .. } => Some(
-                Span::raw(name)
-                    .fg(color_theme.list_ref_remote_branch_fg)
-                    .bold(),
-            ),
-            Ref::Tag { name, .. } => Some(Span::raw(name).fg(color_theme.list_ref_tag_fg).bold()),
             Ref::Stash { .. } => None,
         })
+        .map(|(name, fg)| {
+            let spans = refs_matches
+                .get(name)
+                .map(|pos| highlighted_spans(name.to_string(), pos.clone(), fg, color_theme, false))
+                .unwrap_or_else(|| vec![Span::raw(name).fg(fg).bold()]);
+            (spans, name)
+        })
         .collect();
 
     let mut spans = vec![Span::raw("(").fg(color_theme.list_ref_paren_fg).bold()];
@@ -809,13 +991,14 @@ fn refs_spans<'a>(
         }
     }
 
-    for (i, s) in ref_spans.into_iter().enumerate() {
+    for (i, ss) in ref_spans.into_iter().enumerate() {
+        let (ref_spans, ref_name) = ss;
         if let Head::Branch { name } = head {
-            if s.content == std::borrow::Cow::Borrowed(name) {
+            if ref_name == name {
                 spans.push(Span::raw("HEAD -> ").fg(color_theme.list_head_fg).bold());
             }
         }
-        spans.push(s);
+        spans.extend(ref_spans);
         if i < refs.len() - 1 {
             spans.push(Span::raw(", ").fg(color_theme.list_ref_paren_fg).bold());
         }
@@ -838,7 +1021,7 @@ fn highlighted_spans(
     truncate: bool,
 ) -> Vec<Span<'static>> {
     let mut hm = highlight_matched_text(s)
-        .matched_range(pos.start, pos.end)
+        .matched_indices(pos.matched_indices)
         .not_matched_style(Style::default().fg(base_fg))
         .matched_style(
             Style::default()
